// -*- Socket Streams -*-

// Copyright (C) 2003, 2004, 2005 Pedro Lamarão <pedro.lamarao@mndfck.org>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file address.h
 * @brief User-friendly access to address classes.
 * 
 * This header includes de inet, inet6 and local address classes.
 */

/** @defgroup address Network Addressing
 * @brief The network address classes.
 * 
 * Features of the address classes:
 * - constructors for easier initialization
 * - copy constructors and copy operators
 * - sizeof(sockaddr_T) == sizeof(T_address);
 * - T_address* is convertible to sockaddr*
 * 
 * Using an address class:
 * 
 * @code
 * ipv4_address address(ipv4_address::any, htons(80));
 * int socket = ::socket(AF_INET, SOCK_STREAM, 0);
 * ::bind(socket, &address, sizeof(address)):
 * @endcode
 * 
 * Interacting with the @ref listener module:
 * 
 * @code
 * ipv4_address address(ipv4_address::any, htons(80));
 * listener l(sockets_base::ipv4);
 * l.bind(address);
 * @endcode
 */

#ifndef SOCKETSTREAM_ADDRESS_H
#define SOCKETSTREAM_ADDRESS_H 1

#include <socketstream/inet_factory.h>
#include <socketstream/inet6_factory.h>

#ifndef WIN32
# include <socketstream/local_factory.h>
#endif

#endif // SOCKETSTREAM_ADDRESS_H

