// -*- Socket Streams -*-

// Copyright (C) 2003, 2004, 2005 Pedro Lamarão <pedro.lamarao@mndfck.org>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file local_factory.h
 * @brief Contains the Local address class.
 */

#ifndef SOCKETSTREAM_LOCAL_ADRESS_H
#define SOCKETSTREAM_LOCAL_ADDRESS_H 1

#include <socketstream/config.h>

#include <sys/socket.h>
#include <sys/un.h> // for struct sockaddr_un

namespace ss {

  /** @ingroup address
   * @brief Local address class.
   *
   * This implementation of the address interface deal with Local "addresses",
   * which are really paths in the filesystem.
   */
  class local_address
  : public sockaddr_un
  {
  public:

    typedef sockaddr_un sockaddr_t;

	static const int AF = AF_LOCAL;

    /**
     * @brief Constructor.
     */
    local_address ();

    /**
     * @brief Constructor.
     */
    local_address (char const* __path);

	/**
	 * @brief Copy constructor.
	 */
	local_address (local_address const& that);

	/**
	 * @brief Copy constructor.
	 */
	local_address (sockaddr_un const& that);

	/**
	 * @brief Copy constructor.
	 * 
	 * Checks if address structure is of the correct family.
	 */
	local_address (sockaddr const& that);

	/**
	 * @brief Assignment operator.
	 */
	local_address& operator= (local_address const& that);

	/**
	 * @brief Assignment operator.
	 */
	local_address& operator= (sockaddr_un const& that);

	/**
	 * @brief Assignment operator.
	 * 
	 * Checks if address structure is of the correct family.
	 */
	local_address& operator= (sockaddr const& that);

  };

}

#endif // SOCKETSTREAM_LOCAL_ADDRESS_H
