// -*- Socket Streams -*-

// Copyright (C) 2003, 2004, 2005 Pedro Lamarão <pedro.lamarao@mndfck.org>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file socket.h
 * @brief User-friendly access to basic_socket class.
 */

/** @defgroup socket Socket
 * @brief The socket class
 * 
 * The socket class is a simple wrapper over the socket descriptor.
 * 
 * Features of the socket class:
 * - exception thrown on error condition
 * - overloaded function name "send" and "receive"
 * - default parameters can be omitted for brevity
 * 
 * Using the socket class:
 * 
 * @code
 * extern sockaddr* address;
 * char[1024] buffer;
 * socket s(AF_INET, SOCK_STREAM);
 * s.connect(address, sizeof(sockaddr_in));
 * s.read(buffer, 1024);
 * s.shutdown();
 * @endcode
 * 
 * Note that this socket class is an exposition of the operating system
 * primitives; the true generic representation of a communication stream
 * is the basic_socketbuf template class.
 */

#ifndef SOCKETSTREAM_SOCKET_H
#define SOCKETSTREAM_SOCKET_H 1

#include <socketstream/basic_socket.h>

#endif // SOCKETSTREAM_SOCKET_H

