// -*- Socket Streams -*-

// Copyright (C) 2003, 2004, 2005 Pedro Lamarão <pedro.lamarao@mndfck.org>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file sockets_exception.h
 * @brief Contains the exception class.
 */

#ifndef SOCKETSTREAM_SOCKETS_EXCEPTION_H
#define SOCKETSTREAM_SOCKETS_EXCEPTION_H 1

#include <socketstream/config.h>

#include <exception>

namespace ss {

  /** @ingroup exception
   * @brief Base class for all library exceptions.
   * 
   * This is the base class for all exceptions thrown by the socketstream library.
   * It is derived from std::exception.
   */
  class SOCKETSTREAM_API exception
  : public std::exception
  {
  public:

    virtual
    ~exception () throw () {}

    /**
     * @brief Returns a C-style character string describing
     * the cause of the current error.
     */
    virtual const char*
    what () const throw () { return 0; }

    /**
     * @brief Returns the error code relative to the error condition caught.
     */
    virtual int
    code () const throw () { return 0; }

  };
}

#endif // SOCKETSTREAM_SOCKETS_EXCEPTION_H
