// -*- Socket Streams -*-

// Copyright (C) 2003, 2004, 2005 Pedro Lamarão <pedro.lamarao@mndfck.org>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file basic_socket.cc
 * @doctodo
 */

#include <socketstream/basic_socket.h>

namespace ss {

  typedef basic_socket::descriptor_t descriptor_t;

  basic_socket::basic_socket () : _M_sd()
  {
	  _M_sd = ::socket(sockets_base::def::family, sockets_base::def::style, sockets_base::def::protocol);
    if (_M_sd == -1)
      throw socket_exception();
  }

  basic_socket::basic_socket (int __family, int __style, int __protocol)
  : _M_sd()
  {
    _M_sd = ::socket(__family, __style, __protocol);
    if (_M_sd == -1)
      throw socket_exception();
  }

  basic_socket::basic_socket (descriptor_t __sd)
  : _M_sd(__sd)
  { }

  basic_socket::~basic_socket ()
  {
    ::shutdown(_M_sd, SHUT_RDWR);
    ::close(_M_sd);
  }

  void
  basic_socket::bind (sockaddr const* __addr, socklen_t __size)
  {
    int __r = ::bind(_M_sd, __addr, __size);
    if (__r == -1)
      throw socket_exception();
  }

  void
  basic_socket::connect (sockaddr const* __addr, socklen_t __size)
  {
    // add check for family consistency
    int __r = ::connect(_M_sd, __addr, __size);
    if (__r == -1)
      throw socket_exception();
  }

  void
  basic_socket::listen (int __backlog)
  {
	  int __r = ::listen(_M_sd, __backlog);
    if (__r == -1)
      throw socket_exception();
  }

  descriptor_t
  basic_socket::accept ()
  {
    descriptor_t __sd = ::accept(_M_sd, 0, 0);
    if (__sd == -1)
      throw socket_exception();
    return __sd;
  }

  int
  basic_socket::receive (char* __s, size_t __n, int __flags)
  {
    int __r = ::recv(_M_sd, __s, __n, __flags);
    if (__r == -1)
      throw socket_exception();
    return __r;
  }

  int
  basic_socket::receive (char* __s, size_t __n,
                         sockaddr* __addr, socklen_t* __size, int __flags)
  {
    int __r = ::recvfrom(_M_sd, __s, __n, __flags, __addr, __size);
    if (__r == -1)
      throw socket_exception();
    return __r;
  }

  int
  basic_socket::send (char const* __s, size_t __n, int __flags)
  {
    int __r = ::send(_M_sd, __s, __n, __flags);
    if (__r == -1)
      throw socket_exception();
    return __r;
  }

  int
  basic_socket::send (char const* __s, size_t __n,
                      sockaddr const* __addr, socklen_t __size, int __flags)
  {
    int __r = ::sendto(_M_sd, __s, __n, __flags, __addr, __size);
    if (__r == -1)
      throw socket_exception();
    return __r;
  }

  void
  basic_socket::get_option (int __level, int __option, char* __value,
                            socklen_t* __size) const
  {
    int __r = ::getsockopt(_M_sd, __level, __option, __value, __size);
    if (__r == -1)
      throw socket_exception();
  }

  void
  basic_socket::set_option (int __level, int __option, char const* __value,
                            socklen_t __size)
  {
    int __r = ::setsockopt(_M_sd, __level, __option, __value, __size);
    if (__r == -1)
      throw socket_exception();
  }

  void
  basic_socket::shutdown (int __mode)
  {
    int __m = __mode;
    int __r = ::shutdown(_M_sd, __m);
    if (__r == -1)
      throw socket_exception();
  }

  void
  basic_socket::peer_address (sockaddr* __name, socklen_t* __size) const
  {
    // should we no-op if !is_open() ?
    int __r = ::getpeername(_M_sd, __name, __size);
    if (__r == -1)
      throw socket_exception();
  }

  void
  basic_socket::socket_address (sockaddr* __name, socklen_t* __size) const
  {
    int __r = ::getsockname(_M_sd, __name, __size);
    if (__r == -1)
      throw socket_exception();
  }

}
