// -*- Socket Streams -*-

// Copyright (C) 2003, 2004, 2005 Pedro Lamarão <pedro.lamarao@mndfck.org>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file inet6_factory.cc
 * @doctodo
 */

#include <socketstream/inet6_factory.h>

#include <cstring>

namespace ss {

	in6_addr const& ipv6_address::any = in6addr_any;
	
	in6_addr const& ipv6_address::loopback = in6addr_loopback;
	
	ipv6_address::ipv6_address ()
	{
		this->sin6_family = AF_INET6;
		this->sin6_port = 0;
		this->sin6_flowinfo = 0;
		std::memset(this->sin6_addr.s6_addr, 0, 16);
	}

	ipv6_address::ipv6_address (in6_addr const& address, unsigned short port)
	{
		this->sin6_family = AF_INET6;
		this->sin6_port = htons(port);
		this->sin6_flowinfo = 0;
		std::memcpy(this->sin6_addr.s6_addr, address.s6_addr, 16);
	}

	ipv6_address::ipv6_address (ipv6_address const& that)
	{
		this->sin6_family = AF_INET6;
		(*this) = that;
	}

	ipv6_address::ipv6_address (sockaddr_in6 const& that)
	{
		this->sin6_family = AF_INET6;
		(*this) = that;
	}

	ipv6_address::ipv6_address (struct sockaddr const& that)
	{
		this->sin6_family = AF_INET6;
		if (that.sa_family == AF_INET6)
		{
			sockaddr_in6 const* tmp = reinterpret_cast<sockaddr_in6 const*>(&that);
			(*this) = (*tmp);
		}
		else
		{
			// Error: address of wrong family.
		}
	}

	ipv6_address& ipv6_address::operator= (ipv6_address const& that)
	{
		this->sin6_port = that.sin6_port;
		this->sin6_flowinfo = that.sin6_flowinfo;
		std::memcpy(this->sin6_addr.s6_addr, that.sin6_addr.s6_addr, 16);
		return *this;
	}

	ipv6_address& ipv6_address::operator= (sockaddr_in6 const& that)
	{
		this->sin6_port = that.sin6_port;
		this->sin6_flowinfo = that.sin6_flowinfo;
		std::memcpy(this->sin6_addr.s6_addr, that.sin6_addr.s6_addr, 16);
		return *this;
	}

	ipv6_address& ipv6_address::operator= (struct sockaddr const& that)
	{
		if (that.sa_family == AF_INET6)
		{
			sockaddr_in6 const* tmp = reinterpret_cast<sockaddr_in6 const*>(&that);
			(*this) = (*tmp);
		}
		else
		{
			// Error: address of wrong type.
		}
		return *this;
	}



}
