// -*- Socket Streams -*-

// Copyright (C) 2003, 2004, 2005 Pedro Lamarão <pedro.lamarao@mndfck.org>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file inet_factory.cc
 * @doctodo
 */

#include <cstring>
#include <socketstream/inet_factory.h>

namespace ss {

// The "inline" version of htonl won't work for us here.
#undef htonl

	const in_addr ipv4_address::any = { htonl(INADDR_ANY) };

	const in_addr ipv4_address::loopback = { htonl(INADDR_LOOPBACK) };

	ipv4_address::ipv4_address ()
	{
		this->sin_family = AF_INET;
		this->sin_addr.s_addr = 0;
		this->sin_port = 0;
	}

	ipv4_address::ipv4_address (in_addr const& address, unsigned short port)
	{
		this->sin_family = AF_INET;
		this->sin_addr.s_addr = address.s_addr;
		this->sin_port = port;
	}

	ipv4_address::ipv4_address (ipv4_address const& that)
	{
		this->sin_family = AF_INET;
		(*this) = that;
	}

	ipv4_address::ipv4_address (sockaddr_in const& that)
	{
		this->sin_family = AF_INET;
		(*this) = that;
	}

	ipv4_address::ipv4_address (struct sockaddr const& that)
	{
		this->sin_family = AF_INET;
		if (that.sa_family == AF_INET)
		{
			sockaddr_in const* tmp = reinterpret_cast<sockaddr_in const*>(&that);
			(*this) = (*tmp);
		}
		else
		{
			// Error: address of wrong type.
		}
	}

	ipv4_address& ipv4_address::operator= (ipv4_address const& that)
	{
		this->sin_addr.s_addr = that.sin_addr.s_addr;
		this->sin_port = that.sin_port;
		return *this;
	}

	ipv4_address& ipv4_address::operator= (sockaddr_in const& that)
	{
		this->sin_addr.s_addr = that.sin_addr.s_addr;
		this->sin_port = that.sin_port;
		return *this;
	}

	ipv4_address& ipv4_address::operator= (struct sockaddr const& that)
	{
		if (that.sa_family == AF_INET)
		{
			sockaddr_in const* tmp = reinterpret_cast<sockaddr_in const*>(&that);
			(*this) = (*tmp);
		}
		else
		{
			// Error: address of wrong type.
		}
		return *this;
	}

}
