// -*- Socket Streams -*-

// Copyright (C) 2003, 2004, 2005 Pedro Lamarão <pedro.lamarao@mndfck.org>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file local_factory.cc
 * @doctodo
 */

#include <socketstream/local_factory.h>

#include <cstring>

#ifndef UNIX_PATH_MAX
# define UNIX_PATH_MAX 108
#endif

namespace ss {

  local_address::local_address ()
  {
    this->sun_family = AF_LOCAL;
  }

  local_address::local_address (char const* __path)
  {
    this->sun_family = AF_LOCAL;
    std::strcpy(this->sun_path, __path);
  }
  
  local_address::local_address (local_address const& that)
  {
  	this->sun_family = AF_LOCAL;
  	std::strcpy(this->sun_path, that.sun_path);
  }
  
  local_address::local_address (sockaddr_un const& that)
  {
  	this->sun_family = AF_LOCAL;
  	std::strcpy(this->sun_path, that.sun_path);
  }
  
  local_address::local_address (sockaddr const& that)
  {
  	this->sun_family = AF_LOCAL;
  	if (that.sa_family == AF_LOCAL)
  	{
  		sockaddr_un const* tmp = reinterpret_cast<sockaddr_un const*>(&that);
  		std::strcpy(this->sun_path, tmp->sun_path);
  	}
  	else
  	{
  		// Error: address of wrong family.
  	}
  }
  
  local_address& local_address::operator= (local_address const& that)
  {
  	std::strcpy(this->sun_path, that.sun_path);
  	return *this;
  }
  
  local_address& local_address::operator= (sockaddr_un const& that)
  {
  	std::strcpy(this->sun_path, that.sun_path);
  	return *this;
  }
  
  local_address& local_address::operator= (sockaddr const& that)
  {
  	if (that.sa_family == AF_LOCAL)
  	{
  		sockaddr_un const* tmp = reinterpret_cast<sockaddr_un const*>(&that);
  		std::strcpy(this->sun_path, tmp->sun_path);
  	}
  	else
  	{
  		// Error: address of wrong family.
  	}
  	return *this;
  }

}
