// -*- Socket Streams -*-

// Copyright (C) 2003, 2004, 2005 Pedro Lamarão <pedro.lamarao@mndfck.org>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file resolver.cc
 * @doctodo
 */

#include <socketstream/resolver.h>

namespace ss
{

	resolver::resolver () : _M_hint(0), _M_result(0) {}
	
	resolver::~resolver ()
	{
		if (_M_hint)
			delete _M_hint;
	}
	
	resolver& resolver::family (sockets_base::family value)
	{
		this->initialize_hint();
		_M_hint->ai_family = static_cast<int>(value);
		return *this;
	}
	
	resolver& resolver::style (sockets_base::style value)
	{
		this->initialize_hint();
		_M_hint->ai_socktype = static_cast<int>(value);
		return *this;
	}
	
	resolver& resolver::flags (int value)
	{
		this->initialize_hint();
		_M_hint->ai_flags = value;
		return *this;
	}
	
	void resolver::reset ()
	{
		this->reset_hint();
	}
	
	void resolver::resolve (char const* host, char const* service)
	{
		addrinfo* a;
		int r = ::getaddrinfo(host, service, _M_hint, &a);
		if ( r != 0 )
			throw resolver_exception(r);
		_M_result = resolver_result(a);
	}
	
	void resolver::initialize_hint ()
	{
		if (_M_hint)
			return;
		_M_hint = new addrinfo;
		this->reset_hint();
	}
	
	void resolver::reset_hint ()
	{
		if (!_M_hint)
			return;
		_M_hint->ai_flags    = 0;
		_M_hint->ai_family   = 0;
		_M_hint->ai_socktype = 0;
		_M_hint->ai_protocol = 0;
	}

}
