// -*- Socket Streams -*-

// Copyright (C) 2003, 2004, 2005 Pedro Lamarão <pedro.lamarao@mndfck.org>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file sockets_base.cc
 * @doctodo
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#ifdef WIN32
# define SHUT_RD SD_RECEIVE
# define SHUT_WR SD_SEND
# define SHUT_RDWR SD_BOTH
#else
# include <netinet/in.h>
#endif

#include <socketstream/sockets_base.h>

namespace ss {

  typedef sockets_base::family family;

  const family sockets_base::ipv4    = family(PF_INET);
  const family sockets_base::ipv6    = family(PF_INET6);
#ifndef WIN32
  const family sockets_base::local   = family(PF_LOCAL);
#else
  const family sockets_base::local   = family(PF_UNSPEC);
#endif
#if HAVE_DECL_PF_NETLINK
  const family sockets_base::netlink = family(PF_NETLINK);
#else
  const family sockets_base::netlink = family(PF_UNSPEC);
#endif
#if HAVE_DECL_PF_PACKET
  const family sockets_base::packet  = family(PF_PACKET);
#else
  const family sockets_base::packet  = family(PF_UNSPEC);
#endif

  typedef sockets_base::style style;

  const style sockets_base::stream    = style(SOCK_STREAM);
  const style sockets_base::datagram  = style(SOCK_DGRAM);
  const style sockets_base::raw       = style(SOCK_RAW);
#if HAVE_DECL_SOCK_RDM
  const style sockets_base::rdm       = style(SOCK_RDM);
#else
  const style sockets_base::rdm       = style(0);
#endif
#if HAVE_DECL_SOCK_SEQPACKET
  const style sockets_base::seqpacket = style(SOCK_SEQPACKET);
#else
  const style sockets_base::seqpacket = style(0);
#endif

  typedef sockets_base::protocol protocol;
  
  typedef sockets_base::shutmode shutmode;
  
  const shutmode sockets_base::r  = shutmode(SHUT_RD);
  const shutmode sockets_base::w  = shutmode(SHUT_WR);
  const shutmode sockets_base::rw = shutmode(SHUT_RDWR);
  
  typedef sockets_base::flag flag;

  const flag sockets_base::oob          = flag(MSG_OOB);
  const flag sockets_base::dontroute    = flag(MSG_DONTROUTE);
#if HAVE_DECL_MSG_DONTWAIT
  const flag sockets_base::dontwait     = flag(MSG_DONTWAIT);
#else
  const flag sockets_base::dontwait     = flag(0);
#endif
#if HAVE_DECL_MSG_NOSIGNAL
  const flag sockets_base::nosignal     = flag(MSG_NOSIGNAL);
#else
  const flag sockets_base::nosignal     = flag(0);
#endif
  const flag sockets_base::peek         = flag(MSG_PEEK);
#if HAVE_DECL_MSG_WAITALL
  const flag sockets_base::waitall      = flag(MSG_WAITALL);
#else
  const flag sockets_base::waitall      = flag(0);
#endif
#if HAVE_DECL_MSG_TRUNC
  const flag sockets_base::trunc        = flag(MSG_TRUNC);
#else
  const flag sockets_base::trunc        = flag(0);
#endif
#if HAVE_DECL_MSG_ERRQUEUE
  const flag sockets_base::errqueue     = flag(MSG_ERRQUEUE);
#else
  const flag sockets_base::errqueue     = flag(0);
#endif

  typedef sockets_base::def def;

  const family   def::family   = sockets_base::ipv4;
  const style    def::style    = sockets_base::stream;
  const protocol def::protocol = sockets_base::protocol(0);
  const flag     def::flag     = sockets_base::nosignal;
  
}
